/*

    MiniMono Example 1 for Linux

    (C) Eugene Karataev

    http://www.minimdb.com
    support@minimdb.com


    Example demonstrates how to load, initialize and free
    MiniMono virtual machine and evaluate MUMPS expressions

*/

#include <stdlib.h>
#include <stdio.h>
#include <fcntl.h>  
#include <unistd.h>
#include <dlfcn.h>
#include <signal.h>
#include <strings.h>
#define stricmp strcasecmp

#include "minimono.h"

// virtual machine settings
MINIMONOVM vm;

int main(int argc, char* argv[])
{
  printf( "MiniMono example 1\n");

  freeminimonovm_t        FreeMiniMonoVM;
  createminimonovm_t      CreateMiniMonoVM;
  getdefaultsettingsvm_t  GetDefaultSettingsVM;
  int ret;
  MINIM_STR expr;
  MINIM_STR res;

  // dynamic load MiniMono DLL
  void* hModule = dlopen( "./libminimono.so", RTLD_LAZY);
  if( !hModule)
  {
    printf( "dlopen failed\n");
    goto exit;
  }

  // import functions by name
  CreateMiniMonoVM =
    (createminimonovm_t)dlsym( hModule, "CreateMiniMonoVM");
  if( !CreateMiniMonoVM)
  {
    printf( "Function CreateMiniMonoVM not found\n");
    goto exit;
  }

  FreeMiniMonoVM =
    (freeminimonovm_t)dlsym( hModule, "FreeMiniMonoVM");
  if( !FreeMiniMonoVM)
  {
    printf( "Function FreeMiniMonoVM not found\n");
    goto exit;
  }

  GetDefaultSettingsVM =
    (getdefaultsettingsvm_t)dlsym( hModule, "GetDefaultSettingsVM");
  if( !GetDefaultSettingsVM)
  {
    printf( "Function GetDefaultSettingsVM not found\n");
    goto exit;
  }

  // get default MiniMono settings
  GetDefaultSettingsVM( &vm);

  // assign custom settings for this application
  vm.DataFile = "empty.dat";    // assign datafile name you are using
  vm.DBCacheSize = 100;         // 100 Mbytes
  vm.JournalingEnabled = 0;     // this example does not require journaling

  ret = CreateMiniMonoVM( &vm);
  switch( ret)
  {
    case MINIMONO_SUCCESS:
      printf( "CreateMiniMonoVM: MiniMono virtual machine created\n");
      break;
    case MINIMONO_CREATED:
      printf( "CreateMiniMonoVM: MiniMono virtual machine already been created\n");
      goto exit;
    default:
      printf( "Function CreateMiniMonoVM failed\n");
      goto exit;
  }

  // call MiniMono virtual machine to evaluate expressions

  // $zversion
  expr.len = sprintf( (char*)expr.data, "$zversion");
  vm.cbfunc->Eval( &expr, &res);
  // this result always is string
  // if evaluated expression can be a number, we must call
  // convert function vm.cbfunc->GetStr() to get symbol representation
  printf( "Value of $zversion: %.*s\n", res.len, res.data);

  // $zdate($h,3)
  expr.len = sprintf( (char*)expr.data, "$zdate($h,3)");
  vm.cbfunc->Eval( &expr, &res);
  printf( "Now is: %.*s\n", res.len, res.data);

  // free virtual machine
  FreeMiniMonoVM();

exit:

  if( hModule)
  {
    dlclose( hModule);
  }

  printf( "Example finished, press any key...\n");
  getchar();
  return 0;
};
