/*

    MiniMono Example 4 for Linux

    (C) Eugene Karataev

    http://www.minimdb.com
    support@minimdb.com


    Example demonstrates how to setup some of DLL device event handlers

*/

#include <stdlib.h>
#include <stdio.h>
#include <fcntl.h>  
#include <unistd.h>
#include <dlfcn.h>
#include <signal.h>
#include <strings.h>
#define stricmp strcasecmp

#include "minimono.h"

// virtual machine settings
static MINIMONOVM vm;

// virtual machine device event handlers
static int STDCALL DevWriteStr( ZDLLCB* cbfunc, MINIM_STR* str);
static int STDCALL DevWriteChar( ZDLLCB* cbfunc, int _char);
static int STDCALL DevWriteNL( ZDLLCB* cbfunc);
static int STDCALL DevWriteFF( ZDLLCB* cbfunc);
static int STDCALL DevWriteTab( ZDLLCB* cbfunc, int tabcount);

int main(int argc, char* argv[])
{
  printf( "MiniMono example 4\n");

  freeminimonovm_t        FreeMiniMonoVM;
  createminimonovm_t      CreateMiniMonoVM;
  getdefaultsettingsvm_t  GetDefaultSettingsVM;
  int ret;
  MINIM_STR command;

  // dynamic load MiniMono DLL
  void* hModule = dlopen( "./libminimono.so", RTLD_LAZY);
  if( !hModule)
  {
    printf( "dlopen failed\n");
    goto exit;
  }

  // import functions by name
  CreateMiniMonoVM =
    (createminimonovm_t)dlsym( hModule, "CreateMiniMonoVM");
  if( !CreateMiniMonoVM)
  {
    printf( "Function CreateMiniMonoVM not found\n");
    goto exit;
  }

  FreeMiniMonoVM =
    (freeminimonovm_t)dlsym( hModule, "FreeMiniMonoVM");
  if( !FreeMiniMonoVM)
  {
    printf( "Function FreeMiniMonoVM not found\n");
    goto exit;
  }

  GetDefaultSettingsVM =
    (getdefaultsettingsvm_t)dlsym( hModule, "GetDefaultSettingsVM");
  if( !GetDefaultSettingsVM)
  {
    printf( "Function GetDefaultSettingsVM not found\n");
    goto exit;
  }

  // get default MiniMono settings
  GetDefaultSettingsVM( &vm);

  // assign custom settings for this application
  vm.DataFile = "empty.dat";    // assign datafile name you are using
  vm.DBCacheSize = 100;         // 100 Mbytes
  vm.JournalingEnabled = 0;     // this example does not require journaling

  // assign DLL device event handlers
  vm.WriteStr = DevWriteStr;
  vm.WriteChar = DevWriteChar;
  vm.WriteNL = DevWriteNL;
  vm.WriteFF = DevWriteFF; 
  vm.WriteTab = DevWriteTab;

  ret = CreateMiniMonoVM( &vm);
  switch( ret)
  {
    case MINIMONO_SUCCESS:
      printf( "CreateMiniMonoVM: MiniMono virtual machine created\n");
      break;
    case MINIMONO_CREATED:
      printf( "CreateMiniMonoVM: MiniMono virtual machine already been created\n");
      goto exit;
    default:
      printf( "Function CreateMiniMonoVM failed\n");
      goto exit;
  }

  // call MiniMono virtual machine to execute commands

  // execute device commands
  command.len = sprintf( (char*)command.data, "write \"string\",*46,?12,!,#");
  vm.cbfunc->Execute( &command);

  // free virtual machine
  FreeMiniMonoVM();

exit:

  if( hModule)
  {
    dlclose( hModule);
  }

  printf( "Example finished, press any key...\n");
  getchar();
  return 0;
};

// virtual machine device event handlers
static int STDCALL DevWriteStr( ZDLLCB* cbfunc, MINIM_STR* str)
{
  MINIM_STR _str;
  cbfunc->GetStr( str, &_str);
  printf( "Fired WriteStr(\"%.*s\") event\n", _str.len, _str.data);
  return 0;
};

static int STDCALL DevWriteChar( ZDLLCB* cbfunc, int _char)
{
  printf( "Fired WriteChar(%d) event\n", _char);
  return 0;
};

static int STDCALL DevWriteNL( ZDLLCB* cbfunc)
{
  printf( "Fired WriteNL event\n");
  return 0;
};

static int STDCALL DevWriteFF( ZDLLCB* cbfunc)
{
  printf( "Fired WriteFF event\n");
  return 0;
};

static int STDCALL DevWriteTab( ZDLLCB* cbfunc, int tabcount)
{
  printf( "Fired WriteTab(%d) event\n", tabcount);
  return 0;
};

