/*

    MiniMono Example 6 for Linux

    (C) Eugene Karataev

    http://www.minimdb.com
    support@minimdb.com


    Example demonstrates how to create routine and call it

*/

#include <stdlib.h>
#include <stdio.h>
#include <fcntl.h>  
#include <unistd.h>
#include <dlfcn.h>
#include <signal.h>
#include <strings.h>
#define stricmp strcasecmp

#include "minimono.h"

// virtual machine settings
MINIMONOVM vm;

// virtual machine device event handlers
static int STDCALL DevWriteStr( ZDLLCB* cbfunc, MINIM_STR* str);
static int STDCALL DevWriteChar( ZDLLCB* cbfunc, int _char);
static int STDCALL DevWriteNL( ZDLLCB* cbfunc);
static int STDCALL DevWriteFF( ZDLLCB* cbfunc);
static int STDCALL DevWriteTab( ZDLLCB* cbfunc, int tabcount);

static char* database_name = "%SYS";
static char* global_name = "ROUTINE";
static char* routine_name = "Example6";
// simple routine content
static char* routine_text[] =
{
  "Func(arg)",
  " w \"arg = \",arg,!",
  " n expr=$zd($h,8)",
  " q expr"
};

int main(int argc, char* argv[])
{
  printf( "MiniMono example 6\n");

  freeminimonovm_t        FreeMiniMonoVM;
  createminimonovm_t      CreateMiniMonoVM;
  getdefaultsettingsvm_t  GetDefaultSettingsVM;
  int ret;
  MINIM_STR index_value[ 2];
  MINIM_STR* index[ 2];
  MINIM_STR expr;
  MINIM_STR value;
  MINIM_STR str;
  int i;

  // dynamic load MiniMono DLL
  void* hModule = dlopen( "./libminimono.so", RTLD_LAZY);
  if( !hModule)
  {
    printf( "dlopen failed\n");
    goto exit;
  }

  // import functions by name
  CreateMiniMonoVM =
    (createminimonovm_t)dlsym( hModule, "CreateMiniMonoVM");
  if( !CreateMiniMonoVM)
  {
    printf( "Function CreateMiniMonoVM not found\n");
    goto exit;
  }

  FreeMiniMonoVM =
    (freeminimonovm_t)dlsym( hModule, "FreeMiniMonoVM");
  if( !FreeMiniMonoVM)
  {
    printf( "Function FreeMiniMonoVM not found\n");
    goto exit;
  }

  GetDefaultSettingsVM =
    (getdefaultsettingsvm_t)dlsym( hModule, "GetDefaultSettingsVM");
  if( !GetDefaultSettingsVM)
  {
    printf( "Function GetDefaultSettingsVM not found\n");
    goto exit;
  }

  // get default MiniMono settings
  GetDefaultSettingsVM( &vm);

  // assign custom settings for this application
  vm.DataFile = "empty.dat";    // assign datafile name you are using
  vm.DBCacheSize = 100;         // 100 Mbytes
  vm.JournalingEnabled = 0;     // this example does not require journaling

  // assign DLL device event handlers
  vm.WriteStr = DevWriteStr;
  vm.WriteChar = DevWriteChar;
  vm.WriteNL = DevWriteNL;
  vm.WriteFF = DevWriteFF; 

  ret = CreateMiniMonoVM( &vm);
  switch( ret)
  {
    case MINIMONO_SUCCESS:
      printf( "CreateMiniMonoVM: MiniMono virtual machine created\n");
      break;
    case MINIMONO_CREATED:
      printf( "CreateMiniMonoVM: MiniMono virtual machine already been created\n");
      goto exit;
    default:
      printf( "Function CreateMiniMonoVM failed\n");
      goto exit;
  }

  // create array of index as array of pointers
  index[ 0] = &index_value[ 0];
  index[ 1] = &index_value[ 1];

  index_value[ 0].len = sprintf( (char*)index_value[ 0].data, "%s", routine_name);

  // kill routine before create new
  vm.cbfunc->KillGlobal( global_name, database_name, 1, index);

  // create routine as a series of direct global sets
  for( int i = 0; i < sizeof( routine_text) / sizeof( routine_text[ 0]); i++)
  {
    vm.cbfunc->SetInt32( i + 1, &index_value[ 1]);
    value.len = sprintf( (char*)value.data, "%s", routine_text[ i]);
    vm.cbfunc->WriteGlobal( global_name, database_name, 2, index, &value);
  }

  // compile routine
  expr.len = sprintf( (char*)expr.data, "$v(\"rou\",\"c\",\"%s\")", routine_name);
  vm.cbfunc->Eval( &expr, &value);

  // evaluate expression
  expr.len = sprintf( (char*)expr.data, "$$Func^%s(\"any data\")", routine_name);
  vm.cbfunc->Eval( &expr, &value);
  vm.cbfunc->GetStr( &value, &str);
  printf( "Function result: %.*s\n", str.len, str.data);

  // free virtual machine
  FreeMiniMonoVM();

exit:

  if( hModule)
  {
    dlclose( hModule);
  }

  printf( "Example finished, press any key...\n");
  getchar();
  return 0;
};

// virtual machine device event handlers
// functions emulates very primitive output to screen

static int STDCALL DevWriteStr( ZDLLCB* cbfunc, MINIM_STR* str)
{
  MINIM_STR _str;
  cbfunc->GetStr( str, &_str);
  printf( "%.*s", _str.len, _str.data);
  return 0;
};

static int STDCALL DevWriteChar( ZDLLCB* cbfunc, int _char)
{
  printf( "%c", _char);
  return 0;
};

static int STDCALL DevWriteNL( ZDLLCB* cbfunc)
{
  printf( "\n");
  return 0;
};

static int STDCALL DevWriteFF( ZDLLCB* cbfunc)
{
  printf( "\n");
  return 0;
};

static int STDCALL DevWriteTab( ZDLLCB* cbfunc, int tabcount)
{
  printf( " ");
  return 0;
};

