/*

    MiniMono, MiniM Embedded Edition

    (C) Eugene Karataev

    Header for the MiniMono DLL

    http://www.minimdb.com
    support@minimdb.com

*/

#ifndef __MINIMONO_H__
#define __MINIMONO_H__

#include "zdll.h"

#ifdef __cplusplus
extern "C" {
#endif

typedef int (STDCALL* dlldevuse_t)( ZDLLCB* cbfunc,
  MINIM_STR** param_pairs, int param_pairs_count);

typedef int (STDCALL* dlldevreadstr_t)( ZDLLCB* cbfunc,
  int len, int timeout, MINIM_STR* result);

typedef int (STDCALL* dlldevreadchar_t)( ZDLLCB* cbfunc,
  int timeout, int* result);

typedef int (STDCALL* dlldevwritestr_t)( ZDLLCB* cbfunc,
  MINIM_STR* str);

typedef int (STDCALL* dlldevwritechar_t)( ZDLLCB* cbfunc,
  int _char);

typedef int (STDCALL* dlldevwritenl_t)( ZDLLCB* cbfunc);

typedef int (STDCALL* dlldevwriteff_t)( ZDLLCB* cbfunc);

typedef int (STDCALL* dlldevwritetab_t)( ZDLLCB* cbfunc,
  int tabcount);

typedef int (STDCALL* dlldevgetx_t)( ZDLLCB* cbfunc,
  int* result);

typedef int (STDCALL* dlldevsetx_t)( ZDLLCB* cbfunc,
  int value);

typedef int (STDCALL* dlldevgetkey_t)( ZDLLCB* cbfunc,
  MINIM_STR* result);

typedef int (STDCALL* dlldevsetkey_t)( ZDLLCB* cbfunc,
  MINIM_STR* value);

typedef int (STDCALL* dlldevzeof_t)( ZDLLCB* cbfunc,
  int* result);

#pragma pack(push,1)
typedef struct tagMINIMONOVM
{
  // file of the database
  char*         DataFile;
  // database in read only mode or write enabled
  int           ReadOnly;
  // is database journaled or not,
  // journaling is required for transactions
  int           JournalingEnabled;
  // size of lock table in megabytes
  int           LockAreaSize;
  // size of routine cache in megabytes
  int           RoutineCacheSize;
  // number of devices can be used
  int           DeviceTableSize;
  // length of device name in bytes
  int           DeviceNameSize;
  // size of global cache in megabytes
  int           DBCacheSize;
  // allow (1) or not (0) null subscripts
  int           NullSubscripts;
  // number of transaction levels allowed
  int           TransactLevelLimit;
  // raise error on end of file or not
  int           TrapOnEof;
  // number of stack frames
  int           FrameCount;
  // size of journal cache in megabytes;
  int           JournalCache;
  // file name with MiniM locale definition or null
  char*         LocaleFileName;
  // size of area for local variables in megabytes
  int           ProcessStorage;

  // Functions to call virtual machine
  // This pointer must be assigned by CreateMiniMonoVM function
  ZDLLCB*             cbfunc;

  // Functions to handle MiniMono DLL principal device |DLL|
  // Functions are the same as ZDLL interface except open and close
  // Open and Close are not supported because it is a principal device
  dlldevuse_t         Use;
  dlldevreadstr_t     ReadStr;
  dlldevreadchar_t    ReadChar;
  dlldevwritestr_t    WriteStr;
  dlldevwritechar_t   WriteChar;
  dlldevwritenl_t     WriteNL;
  dlldevwriteff_t     WriteFF;
  dlldevwritetab_t    WriteTab;
  dlldevgetx_t        GetX;
  dlldevgetx_t        GetY;
  dlldevsetx_t        SetX;
  dlldevsetx_t        SetY;
  dlldevgetkey_t      GetKEY;
  dlldevsetkey_t      SetKEY;
  dlldevzeof_t        GetZEOF;
  dlldevgetkey_t      GetZA;
  dlldevgetkey_t      GetZB;

} MINIMONOVM;
#pragma pack(pop)

// return codes

// action completed successfully
#define MINIMONO_SUCCESS ( 0 )

// virtual machine already was created, failed to create second vm
#define MINIMONO_CREATED ( 1 )

// failed to create virtual machine with this settings
#define MINIMONO_CREATEFAILED ( 2 )

// next functions are exported with the following prototypes and with names:
// "CreateMiniMonoVM"
// "FreeMiniMonoVM"
// "GetDefaultSettingsVM"

// create one virtual machine per process
// int STDCALL CreateMiniMonoVM( MINIMONOVM* init);
typedef int ( STDCALL * createminimonovm_t)( MINIMONOVM* init);

// remove virtual machine
// void STDCALL FreeMiniMonoVM( void);
typedef void ( STDCALL* freeminimonovm_t)( void);

// get default settings
// void STDCALL GetDefaultSettingsVM( MINIMONOVM* init);
typedef void ( STDCALL* getdefaultsettingsvm_t)( MINIMONOVM* init);

// set Ctrl+C interrupt signal
// extern "C" __declspec( dllexport) void STDCALL CtrlBreakVM( int set_break);
typedef void ( STDCALL* ctrlbreakvm_t)( int set_break);

// $listget()
// int STDCALL MNMListGet( MINIM_STR* List, int pos, MINIM_STR* Element);
typedef int ( STDCALL* mnmlistget_t)( MINIM_STR* List,
  int pos, MINIM_STR* Element);

// $list with assignment
// int STDCALL MNMListSet( MINIM_STR* List, int pos, MINIM_STR* Element);
typedef int (STDCALL* mnmlistset_t)( MINIM_STR* List,
  int pos, MINIM_STR* Element);

// $listlength()
// int STDCALL MNMListLength( MINIM_STR* List);
typedef int (STDCALL* mnmlistlength_t)( MINIM_STR* List);

// $zquote()  
// int STDCALL MNMText( MINIM_STR* Source, MINIM_STR* Target);
typedef int ( STDCALL* mnmtext_t)( MINIM_STR* Source, MINIM_STR* Target);

#ifdef __cplusplus
}
#endif

#endif // __MINIMONO_H__