/*

    MiniM

    ZDevice function interface

    Copyright (C) Eugene Karataev
    All rights reserved

*/

#ifndef __ZDEVICE_H__
#define __ZDEVICE_H__

#include "zdll.h"

#ifdef __cplusplus
extern "C" {
#endif

#ifdef STDCALL
#undef STDCALL
#endif
#ifdef _WIN32
#define STDCALL __stdcall
#else
#define STDCALL __attribute__((stdcall))
#endif

typedef void* ZDEVICEHANDLE;

typedef ZDEVICEHANDLE (STDCALL* zdevopen_t)( ZDLLCB* cbfunc,
  const char* dev_name, MINIM_STR** param_pairs, int param_pairs_count,
  int timeout);

/* function's return code to indicate device operation state */  
#define ZDEV_NO_ERROR       ( 0 )
#define ZDEV_ERR_READ       ( 1 )
#define ZDEV_ERR_WRITE      ( 2 )
#define ZDEV_ERR_ENDOFFILE  ( 3 )
#define ZDEV_ERR_DEVICE     ( 4 )

typedef int (STDCALL* zdevclose_t)( ZDLLCB* cbfunc, ZDEVICEHANDLE hDevice,
  MINIM_STR** param_pairs, int param_pairs_count);

typedef int (STDCALL* zdevuse_t)( ZDLLCB* cbfunc, ZDEVICEHANDLE hDevice,
  MINIM_STR** param_pairs, int param_pairs_count);

typedef int (STDCALL* zdevreadstr_t)( ZDLLCB* cbfunc, ZDEVICEHANDLE hDevice,
  int len, int timeout, MINIM_STR* result);

typedef int (STDCALL* zdevreadchar_t)( ZDLLCB* cbfunc, ZDEVICEHANDLE hDevice,
  int timeout, int* result);

typedef int (STDCALL* zdevwritestr_t)( ZDLLCB* cbfunc, ZDEVICEHANDLE hDevice,
  MINIM_STR* str);

typedef int (STDCALL* zdevwritechar_t)( ZDLLCB* cbfunc, ZDEVICEHANDLE hDevice,
  int _char);

typedef int (STDCALL* zdevwritenl_t)( ZDLLCB* cbfunc, ZDEVICEHANDLE hDevice);

typedef int (STDCALL* zdevwriteff_t)( ZDLLCB* cbfunc, ZDEVICEHANDLE hDevice);

typedef int (STDCALL* zdevwritetab_t)( ZDLLCB* cbfunc, ZDEVICEHANDLE hDevice,
  int tabcount);

typedef int (STDCALL* zdevgetx_t)( ZDLLCB* cbfunc, ZDEVICEHANDLE hDevice,
  int* result);

typedef int (STDCALL* zdevsetx_t)( ZDLLCB* cbfunc, ZDEVICEHANDLE hDevice,
  int value);

typedef int (STDCALL* zdevgetkey_t)( ZDLLCB* cbfunc, ZDEVICEHANDLE hDevice,
  MINIM_STR* result);

typedef int (STDCALL* zdevsetkey_t)( ZDLLCB* cbfunc, ZDEVICEHANDLE hDevice,
  MINIM_STR* value);

typedef int (STDCALL* zdevzeof_t)( ZDLLCB* cbfunc, ZDEVICEHANDLE hDevice,
  int* result);

/* zdevice functions interface */
#pragma pack(push,1)
typedef struct _ZDEVICE
{
  zdevopen_t        Open;
  zdevclose_t       Close;
  zdevuse_t         Use;
  zdevreadstr_t     ReadStr;
  zdevreadchar_t    ReadChar;
  zdevwritestr_t    WriteStr;
  zdevwritechar_t   WriteChar;
  zdevwritenl_t     WriteNL;
  zdevwriteff_t     WriteFF;
  zdevwritetab_t    WriteTab;
  zdevgetx_t        GetX;
  zdevgetx_t        GetY;
  zdevsetx_t        SetX;
  zdevsetx_t        SetY;
  zdevgetkey_t      GetKEY;
  zdevsetkey_t      SetKEY;
  zdevzeof_t        GetZEOF;
  zdevgetkey_t      GetZA;
  zdevgetkey_t      GetZB;
} ZDEVICE;
#pragma pack(pop)

/*
    Module should export function with "GetZDevice" name as zdeviceexport_t.
    Function should return pointer to one ZDEVICE structure.
*/
typedef ZDEVICE* ( STDCALL * zdeviceexport_t)( void);

#ifdef __cplusplus
}
#endif

#endif
