/*

    MiniMono Example 3

    (C) Eugene Karataev

    http://www.minimdb.com
    support@minimdb.com


    Example demonstrates how to operate with $list structures
    and how to decorate strings to a MUMPS expression syntax.

*/

class example3
{
	public static void main(String[] args)
	{
		try
		{
			String CodePage = "Cp1251";
			System.out.println("Java example3 how to call MiniMono");

			minimonoj MiniMono = new minimonoj( CodePage);
			minimonoj.MiniMonoVM init = new minimonoj.MiniMonoVM( CodePage);

			MiniMono.GetDefaultSettings( init);

			init.DataFile = "sys.dat";
			init.JournalingEnabled = 0;

			int ret = MiniMono.CreateMiniMono( init);
			System.out.println("MiniMono creation error: " + ret);

			// call virtual machine to create $list structure
			MiniMono.Execute( "s var=$lb($zv,$sy,$zd($h,8))");

			String list = MiniMono.Read( "var");

			// display list length was read de facto
			int n = MiniMono.ListLength( list);
			System.out.println( "Actual length of the list is: " + n);

			// unpack and display list items separately
			for( int i = 1; i <= n; i ++)
			{
				System.out.println( "Item " + i + " : " + MiniMono.ListGet( list, i));
			}

			// create list structure
			// initial value of the list is an empty string
			list = "";
			// set 1 list item
			list = MiniMono.ListSet( list, 1, "123456");
			// set 2 list item
			list = MiniMono.ListSet( list, 2, "123.456");
			// set 3 list item
			list = MiniMono.ListSet( list, 3, "Hello");

			// write to local variable
			MiniMono.Execute( "s varname=" + MiniMono.Text( list));

			// read list items from MUMPS context
			for( int i = 1; i <= 3; i++)
			{
				System.out.println( "List item " + i + " : " + MiniMono.Read( "$lg(varname," + i + ")"));
			}

			// show special symbols decoration
			System.out.println( "Strings decorated with MUMPS expression syntax:");
			System.out.println( "Result: " + MiniMono.Text( "123.456"));
			System.out.println( "Result: " + MiniMono.Text( "Hello, world!"));
			System.out.println( "Result: " + MiniMono.Text( "String\r\nwith line feeds\r\n\tand tabs\t"));

			MiniMono.FreeMiniMono();

		}
		catch(Throwable e)
		{
			e.printStackTrace();
		}
		return;
	};
};