/*

    MiniM

    JNI interface for MiniMono

    Eugene Karataev
    support@minimdb.com

*/

import java.io.File;

public class minimonoj
{
	public static class MiniMonoVM
	{
	  public String CodePage;
	  // file of the database
	  // char*         DataFile;
	  public String DataFile;
	  // database in read only mode or write enabled
	  public int           ReadOnly;
	  // is database journaled or not,
	  // journaling is required for transactions
	  public int           JournalingEnabled;
	  // size of lock table in megabytes
	  public int           LockAreaSize;
	  // size of routine cache in megabytes
	  public int           RoutineCacheSize;
	  // number of devices can be used
	  public int           DeviceTableSize;
	  // length of device name in bytes
	  public int           DeviceNameSize;
	  // size of global cache in megabytes
	  public int           DBCacheSize;
	  // allow (1) or not (0) null subscripts
	  public int           NullSubscripts;
	  // number of transaction levels allowed
	  public int           TransactLevelLimit;
	  // raise error on end of file or not
	  public int           TrapOnEof;
	  // number of stack frames
	  public int           FrameCount;
	  // size of journal cache in megabytes;
	  public int           JournalCache;
	  // file name with MiniM locale definition or null
	  // char*         LocaleFileName;
	  public String LocaleFileName;
	  // size of area for local variables in megabytes
	  public int           ProcessStorage;

	  public MiniMonoVM( String _CodePage)
	  {
		CodePage = _CodePage;
		return;
	  };

	  public void SetDataFile( byte[] _DataFile)
		  throws java.io.UnsupportedEncodingException,
		       java.lang.Exception
	  {
		DataFile = new String( _DataFile, 0, _DataFile.length, CodePage);
	  };
	  byte[] GetDataFile()
		  throws java.io.UnsupportedEncodingException,
		       java.lang.Exception
	  {
		  return DataFile.getBytes( CodePage);
	  };

	  public void SetLocaleFileName( byte[] _LocaleFileName)
		  throws java.io.UnsupportedEncodingException,
		       java.lang.Exception
	  {
		LocaleFileName = new String( _LocaleFileName, 0, _LocaleFileName.length, CodePage);
	  };
	  byte[] GetLocaleFileName()
		  throws java.io.UnsupportedEncodingException,
		       java.lang.Exception
	  {
		  return LocaleFileName.getBytes( CodePage);
	  };
	};

	public String CodePage;
	public final int MINIMSTR_MAX = 32 * 1024 - 1;
	public final int MINIMONO_SUCCESS = 0;
	public final int MINIMONO_CREATED = 1;
	public final int MINIMONO_CREATEFAILED = 2;
	private String strMaxString = "Max string exeeded.";

	static
	{
		String dllname = "minimonoj.dll";
		try
		{
			// first see in PATH
			System.load(dllname);
		}
		catch( Throwable e1)
		{
			// failed, see in current directory
			try
			{
				File dir = new File( ".");
				System.load( dir.getCanonicalPath() + "\\" + dllname );
			}
			catch( Throwable e2)
			{
				e2.printStackTrace();
			}
		}
	}

	public minimonoj( String _CodePage)
		throws java.lang.Exception
	{
		CodePage = _CodePage;
	};
	
	public native void GetDefaultSettings( MiniMonoVM init);

	public native void CtrlBreak( int set_break);

	public native void FreeMiniMono();

	public native int CreateMiniMono( MiniMonoVM init);

	public native byte[] GetLastErrorBin();

	public String GetLastError()
		throws java.io.UnsupportedEncodingException
	{
		byte b[] = GetLastErrorBin();
		return new String( b, CodePage);
	};

	public native int Read( byte Value[], byte Res[]);

	public String Read( String _Value)
		throws java.io.UnsupportedEncodingException,
		       java.lang.Exception
	{
		if( _Value.length() > MINIMSTR_MAX)
		{
			throw new java.lang.Exception( strMaxString);
		}
		byte[] ToRead = _Value.getBytes( CodePage);
		byte[] res = new byte[ MINIMSTR_MAX];
		int ret = Read( ToRead, res);
		if( ret < 0)
		{
			throw new java.lang.Exception( 
				"MiniMono read error: " + GetLastError());
		}
		return new String( res, 0, ret, CodePage);
	};

	public native int ExecuteBin( byte[] Commands);

	public void Execute( String Commands)
		throws java.io.UnsupportedEncodingException,
		       java.lang.Exception
	{
		if( Commands.length() > MINIMSTR_MAX)
		{
			throw new java.lang.Exception( strMaxString);
		}
		int ret = ExecuteBin( Commands.getBytes( CodePage));
		if( ret != 0)
		{
			throw new java.lang.Exception( 
				"MiniMono execute error: " + GetLastError());
		}
		return;
	};

	public native int SetTest( int test_value);

	// override this function
	public int DevWriteStr( String str)
	{
		return 0;
	};

	public int DevWriteStrBin( byte[] data)
		throws java.io.UnsupportedEncodingException,
		       java.lang.Exception
	{
		return DevWriteStr( new String( data, CodePage));
	};

	// override this function
	public int DevUse( int Pos, String _Option, String Value)
	{
		return 0;
	};

	public int DevUseBin( int Pos, byte[] _Option, byte[] _Value)
		throws java.io.UnsupportedEncodingException,
		       java.lang.Exception
	{
		return DevUse( Pos, new String( _Option, CodePage), new String( _Value, CodePage));
	};

	// this structure must be returned on read events
	public static class DevReadResult
	{
		int ReadReturn;  // 0 on success
		byte[] ReadStr;  // bytes was read on DevReadStr
		int ReadChar;    // code was read on DevReadChar
	};
	// override this function
	public DevReadResult DevReadStr( int len, int timeout)
	{
		DevReadResult ret = new DevReadResult();
		ret.ReadReturn = 0;
		return ret;
	};
	// override this function
	public DevReadResult DevReadChar( int timeout)
	{
		DevReadResult ret = new DevReadResult();
		ret.ReadChar = -1;
		ret.ReadReturn = 0;
		return ret;
	};
	// override this function
	public int DevWriteChar( int _Char)
	{
		return 0;
	};
	// override this function
	public int DevWriteNL()
	{
		return 0;
	};
	// override this function
	public int DevWriteFF()
	{
		return 0;
	};
	// override this function
	public int DevWriteTab( int tabcount)
	{
		return 0;
	};
	// this structure must be returned on get state events
	public static class DevState
	{
		int EventReturn;  // 0 on success
		int X;
		int Y;
		byte[] Key;
		int ZEOF;
		byte[] ZA;
		byte[] ZB;
	};
	// override this function
	public DevState DevGetX()
	{
		DevState ret = new DevState();
		ret.EventReturn = 0;
		// assign X here
		ret.X = 0;
		return ret;
	};
	// override this function
	public DevState DevGetY()
	{
		DevState ret = new DevState();
		ret.EventReturn = 0;
		// assign Y here
		ret.Y = 0;
		return ret;
	};
	// override this function
	public int DevSetX( int X)
	{
		return 0;
	};
	// override this function
	public int DevSetY( int Y)
	{
		return 0;
	};
	// override this function
	public int DevSetKey( byte[] Key)
	{
		return 0;
	};
	// override this function
	public DevState DevGetKey()
	{
		DevState ret = new DevState();
		// assign ret.Key here
		ret.EventReturn = 0;
		return ret;
	};
	// override this function
	public DevState DevGetZA()
	{
		DevState ret = new DevState();
		// assign ret.ZA here
		ret.EventReturn = 0;
		return ret;
	};
	// override this function
	public DevState DevGetZB()
	{
		DevState ret = new DevState();
		// assign ret.ZB here
		ret.EventReturn = 0;
		return ret;
	};
	// override this function
	public DevState DevZEOF()
	{
		DevState ret = new DevState();
		// assign ret.ZEOF here
		ret.EventReturn = 0;
		return ret;
	};

	private native int MNMText( byte _Source[], byte _Target[]);

	public String Text( String Source)
		throws java.io.UnsupportedEncodingException,
		       java.lang.Exception
	{
		if( Source.length() > MINIMSTR_MAX)
		{
			throw new java.lang.Exception( strMaxString);
		}
		byte[] source = Source.getBytes( CodePage);
		byte[] res = new byte[ MINIMSTR_MAX];
		int ret = MNMText( source, res);
		if( ret < 0)
		{
			throw new java.lang.Exception( strMaxString);
		}
		return new String( res, 0, ret, CodePage);
	};

	private native int MNMListLength( byte _List[]);
	public int ListLength( String _List)
		throws java.io.UnsupportedEncodingException,
		       java.lang.Exception
	{
		if( _List.length() > MINIMSTR_MAX)
		{
			throw new java.lang.Exception( strMaxString);
		}
		byte[] list = _List.getBytes( CodePage);
		return MNMListLength( list);
	};

	private native int MNMListGet( byte _List[], int Pos, byte Value[]);
	public String ListGet( String _List, int Pos)
		throws java.io.UnsupportedEncodingException,
		       java.lang.Exception
	{
		if( _List.length() > MINIMSTR_MAX)
		{
			throw new java.lang.Exception( strMaxString);
		}
		byte[] list = _List.getBytes( CodePage);
		byte[] res = new byte[ MINIMSTR_MAX];
		int ret = MNMListGet( list, Pos, res);
		if( ret < 0)
		{
			throw new java.lang.Exception( "MiniMono list unpack error.");
		}
		return new String( res, 0, ret, CodePage);
	};

	private native int MNMListSet( byte _List[], int Pos, byte Value[], byte Res[]);
	public String ListSet( String _List, int Pos, String _Element)
		throws java.io.UnsupportedEncodingException,
		       java.lang.Exception
	{
		if( _List.length() > MINIMSTR_MAX)
		{
			throw new java.lang.Exception( strMaxString);
		}
		if( _Element.length() > MINIMSTR_MAX)
		{
			throw new java.lang.Exception( strMaxString);
		}

		byte[] list = _List.getBytes( CodePage);
		byte[] element = _Element.getBytes( CodePage);
		byte[] res = new byte[ MINIMSTR_MAX];
		
		int ret = MNMListSet( list, Pos, element, res);
		if( ret < 0)
		{
			throw new java.lang.Exception( "MiniMono list unpack error.");
		}

		return new String( res, 0, ret, CodePage);
	};

	private void DLLCallbackError( int errcode)
		throws java.lang.Exception
	{
		String ErrText = "";
		// JNI return negative values of error codes ZDLL_CALLBACK_XXX 
		switch( errcode)
		{
			case - 1: ErrText = "Callback Syntax"; break;
			case - 2: ErrText = "Callback Parameters"; break;
			case - 3: ErrText = "Callback Argc"; break;
			case - 4: ErrText = "Callback Undefined"; break;
			case - 5: ErrText = "Callback Common"; break;
			case - 6: 
				throw new java.lang.Exception( "MiniMono Halt");

		}
		if( ErrText == "")
		{
			throw new java.lang.Exception( 
				"MiniMono error: " + GetLastError());
		}
		else
		{
			throw new java.lang.Exception( 
				"MiniMono " + ErrText + " error: " + GetLastError());
		}
	}

	private native int ReadLocalBin( byte[] name, byte[][] Index, byte[] value);
	public String ReadLocal( String Name, String[] _Index)
		throws java.io.UnsupportedEncodingException,
		       java.lang.Exception
	{
		int argc = _Index == null ? 0 : _Index.length;
		byte[][] Index = new byte[ argc][];
		for( int i = 0; i < argc; i++)
		{
			Index[ i] = _Index[ i].getBytes( CodePage);
		}
		byte[] result = new byte[ MINIMSTR_MAX];
		int ret = ReadLocalBin( Name.getBytes( CodePage), Index, result);
		if( ret < 0)
		{
			// JNI return negative values of error codes ZDLL_CALLBACK_XXX 
			DLLCallbackError( ret);
		}
		return new String( result, 0, ret, CodePage);
	};

	private native int WriteLocalBin( byte[] name, byte[][] Index, byte[] value);
	public void WriteLocal( String Name, String[] _Index, String Value)
		throws java.io.UnsupportedEncodingException,
		       java.lang.Exception
	{
		int argc = _Index == null ? 0 : _Index.length;
		byte[][] Index = new byte[ argc][];
		for( int i = 0; i < argc; i++)
		{
			Index[ i] = _Index[ i].getBytes( CodePage);
		}
		byte[] value = Value.getBytes( CodePage);
		int ret = WriteLocalBin( Name.getBytes( CodePage), Index, value);
		if( ret < 0)
		{
			// JNI return negative values of error codes ZDLL_CALLBACK_XXX 
			DLLCallbackError( ret);
		}
		return;
	};

	private native int ReadGlobalBin( byte[] name, byte[][] Index, byte[] value);
	public String ReadGlobal( String Name, String[] _Index)
		throws java.io.UnsupportedEncodingException,
		       java.lang.Exception
	{
		int argc = _Index == null ? 0 : _Index.length;
		byte[][] Index = new byte[ argc][];
		for( int i = 0; i < argc; i++)
		{
			Index[ i] = _Index[ i].getBytes( CodePage);
		}
		byte[] result = new byte[ MINIMSTR_MAX];
		int ret = ReadGlobalBin( Name.getBytes( CodePage), Index, result);
		if( ret < 0)
		{
			// JNI return negative values of error codes ZDLL_CALLBACK_XXX 
			DLLCallbackError( ret);
		}
		return new String( result, 0, ret, CodePage);
	};

	private native int WriteGlobalBin( byte[] name, byte[][] Index, byte[] value);
	public void WriteGlobal( String Name, String[] _Index, String Value)
		throws java.io.UnsupportedEncodingException,
		       java.lang.Exception
	{
		int argc = _Index == null ? 0 : _Index.length;
		byte[][] Index = new byte[ argc][];
		for( int i = 0; i < argc; i++)
		{
			Index[ i] = _Index[ i].getBytes( CodePage);
		}
		byte[] value = Value.getBytes( CodePage);
		int ret = WriteGlobalBin( Name.getBytes( CodePage), Index, value);
		if( ret < 0)
		{
			// JNI return negative values of error codes ZDLL_CALLBACK_XXX 
			DLLCallbackError( ret);
		}
		return;
	};

	private native int UserFuncBin( byte[] name, byte[][] Index, byte[] value);
	public String UserFunc( String Name, String[] _Index)
		throws java.io.UnsupportedEncodingException,
		       java.lang.Exception
	{
		int argc = _Index == null ? 0 : _Index.length;
		byte[][] Index = new byte[ argc][];
		for( int i = 0; i < argc; i++)
		{
			Index[ i] = _Index[ i].getBytes( CodePage);
		}
		byte[] result = new byte[ MINIMSTR_MAX];
		int ret = UserFuncBin( Name.getBytes( CodePage), Index, result);
		if( ret < 0)
		{
			// JNI return negative values of error codes ZDLL_CALLBACK_XXX 
			DLLCallbackError( ret);
		}
		return new String( result, 0, ret, CodePage);
	};

	private native int UserDoBin( byte[] name, byte[][] Index);
	public void UserDo( String Name, String[] _Index)
		throws java.io.UnsupportedEncodingException,
		       java.lang.Exception
	{
		int argc = _Index == null ? 0 : _Index.length;
		byte[][] Index = new byte[ argc][];
		for( int i = 0; i < argc; i++)
		{
			Index[ i] = _Index[ i].getBytes( CodePage);
		}
		int ret = UserDoBin( Name.getBytes( CodePage), Index);
		if( ret < 0)
		{
			// JNI return negative values of error codes ZDLL_CALLBACK_XXX 
			DLLCallbackError( ret);
		}
		return;
	};

	private native int KillLocalBin( byte[] name, byte[][] Index);
	public void KillLocal( String Name, String[] _Index)
		throws java.io.UnsupportedEncodingException,
		       java.lang.Exception
	{
		int argc = _Index == null ? 0 : _Index.length;
		byte[][] Index = new byte[ argc][];
		for( int i = 0; i < argc; i++)
		{
			Index[ i] = _Index[ i].getBytes( CodePage);
		}
		int ret = KillLocalBin( Name.getBytes( CodePage), Index);
		if( ret < 0)
		{
			// JNI return negative values of error codes ZDLL_CALLBACK_XXX 
			DLLCallbackError( ret);
		}
		return;
	};

	private native int KillGlobalBin( byte[] name, byte[][] Index);
	public void KillGlobal( String Name, String[] _Index)
		throws java.io.UnsupportedEncodingException,
		       java.lang.Exception
	{
		int argc = _Index == null ? 0 : _Index.length;
		byte[][] Index = new byte[ argc][];
		for( int i = 0; i < argc; i++)
		{
			Index[ i] = _Index[ i].getBytes( CodePage);
		}
		int ret = KillGlobalBin( Name.getBytes( CodePage), Index);
		if( ret < 0)
		{
			// JNI return negative values of error codes ZDLL_CALLBACK_XXX 
			DLLCallbackError( ret);
		}
		return;
	};

	private native int OrderLocalBin( byte[] name, byte[][] Index, int Direction, byte[] value);
	public String OrderLocal( String Name, String[] _Index, int Direction)
		throws java.io.UnsupportedEncodingException,
		       java.lang.Exception
	{
		int argc = _Index == null ? 0 : _Index.length;
		byte[][] Index = new byte[ argc][];
		for( int i = 0; i < argc; i++)
		{
			Index[ i] = _Index[ i].getBytes( CodePage);
		}
		byte[] result = new byte[ MINIMSTR_MAX];
		int ret = OrderLocalBin( Name.getBytes( CodePage), Index, Direction, result);
		if( ret < 0)
		{
			// JNI return negative values of error codes ZDLL_CALLBACK_XXX 
			DLLCallbackError( ret);
		}
		return new String( result, 0, ret, CodePage);
	};

	private native int OrderGlobalBin( byte[] name, byte[][] Index, int Direction, byte[] value);
	public String OrderGlobal( String Name, String[] _Index, int Direction)
		throws java.io.UnsupportedEncodingException,
		       java.lang.Exception
	{
		int argc = _Index == null ? 0 : _Index.length;
		byte[][] Index = new byte[ argc][];
		for( int i = 0; i < argc; i++)
		{
			Index[ i] = _Index[ i].getBytes( CodePage);
		}
		byte[] result = new byte[ MINIMSTR_MAX];
		int ret = OrderGlobalBin( Name.getBytes( CodePage), Index, Direction, result);
		if( ret < 0)
		{
			// JNI return negative values of error codes ZDLL_CALLBACK_XXX 
			DLLCallbackError( ret);
		}
		return new String( result, 0, ret, CodePage);
	};

	private native int IncLocalBin( byte[] name, byte[][] Index, byte[] value);
	public String IncLocal( String Name, String[] _Index)
		throws java.io.UnsupportedEncodingException,
		       java.lang.Exception
	{
		int argc = _Index == null ? 0 : _Index.length;
		byte[][] Index = new byte[ argc][];
		for( int i = 0; i < argc; i++)
		{
			Index[ i] = _Index[ i].getBytes( CodePage);
		}
		byte[] result = new byte[ MINIMSTR_MAX];
		int ret = IncLocalBin( Name.getBytes( CodePage), Index, result);
		if( ret < 0)
		{
			// JNI return negative values of error codes ZDLL_CALLBACK_XXX 
			DLLCallbackError( ret);
		}
		return new String( result, 0, ret, CodePage);
	};

	private native int IncGlobalBin( byte[] name, byte[][] Index, byte[] value);
	public String IncGlobal( String Name, String[] _Index)
		throws java.io.UnsupportedEncodingException,
		       java.lang.Exception
	{
		int argc = _Index == null ? 0 : _Index.length;
		byte[][] Index = new byte[ argc][];
		for( int i = 0; i < argc; i++)
		{
			Index[ i] = _Index[ i].getBytes( CodePage);
		}
		byte[] result = new byte[ MINIMSTR_MAX];
		int ret = IncGlobalBin( Name.getBytes( CodePage), Index, result);
		if( ret < 0)
		{
			// JNI return negative values of error codes ZDLL_CALLBACK_XXX 
			DLLCallbackError( ret);
		}
		return new String( result, 0, ret, CodePage);
	};

	private native int DataLocalBin( byte[] name, byte[][] Index);
	public int DataLocal( String Name, String[] _Index)
		throws java.io.UnsupportedEncodingException,
		       java.lang.Exception
	{
		int argc = _Index == null ? 0 : _Index.length;
		byte[][] Index = new byte[ argc][];
		for( int i = 0; i < argc; i++)
		{
			Index[ i] = _Index[ i].getBytes( CodePage);
		}
		int ret = DataLocalBin( Name.getBytes( CodePage), Index);
		if( ret < 0)
		{
			// JNI return negative values of error codes ZDLL_CALLBACK_XXX 
			DLLCallbackError( ret);
		}
		return ret;
	};

	private native int DataGlobalBin( byte[] name, byte[][] Index);
	public int DataGlobal( String Name, String[] _Index)
		throws java.io.UnsupportedEncodingException,
		       java.lang.Exception
	{
		int argc = _Index == null ? 0 : _Index.length;
		byte[][] Index = new byte[ argc][];
		for( int i = 0; i < argc; i++)
		{
			Index[ i] = _Index[ i].getBytes( CodePage);
		}
		int ret = DataGlobalBin( Name.getBytes( CodePage), Index);
		if( ret < 0)
		{
			// JNI return negative values of error codes ZDLL_CALLBACK_XXX 
			DLLCallbackError( ret);
		}
		return ret;
	};
};
